@@ -22,7 +22,9 @@
 
 #include <QtCore/QFile>
 #include <QtCore/QStringList>
-#include <QtCore/QTextCodec>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+#    include <QtCore/QTextCodec>
+#endif
 #include <QtCore/QTextStream>
 #include <QtCore/QFileInfo>
 #include <QDebug>
@@ -34,14 +36,7 @@ using namespace KODE;
 class Printer::Private
 {
 public:
-    Private(Printer *parent)
-        : mParent(parent),
-          mCreationWarning(false),
-          mLabelsDefineIndent(true),
-          mIndentLabels(true),
-          mGenerator("libkode")
-    {
-    }
+    Private(Printer *parent) : mParent(parent) {}
 
     void addLabel(Code &code, const QString &label);
     QString classHeader(const Class &classObject, bool publicMembers, bool nestedClass = false);
@@ -52,10 +47,10 @@ public:
 
     Printer *mParent;
     Style mStyle;
-    bool mCreationWarning;
-    bool mLabelsDefineIndent;
-    bool mIndentLabels;
-    QString mGenerator;
+    bool mCreationWarning = false;
+    bool mLabelsDefineIndent = true;
+    bool mIndentLabels = true;
+    QString mGenerator = QStringLiteral("libkode");
     QString mOutputDirectory;
     QString mSourceFile;
     QStringList mStatementsAfterIncludes;
@@ -158,7 +153,13 @@ QString Printer::Private::classHeader(const Class &classObject, bool publicMembe
     if (classObject.isQObject()) {
         code += "Q_OBJECT";
         code.newLine();
+    } else {
+        if (classObject.isQGadget()) {
+            code += "Q_GADGET";
+            code.newLine();
+        }
     }
+
     Q_FOREACH (const QString &declMacro, classObject.declarationMacros()) {
         code += declMacro;
         code.newLine();
@@ -200,7 +201,7 @@ QString Printer::Private::classHeader(const Class &classObject, bool publicMembe
 
         Enum::List::ConstIterator it;
         for (it = enums.constBegin(); it != enums.constEnd(); ++it)
-            code += (*it).declaration();
+            (*it).printDeclaration(code);
 
         if (mLabelsDefineIndent)
             code.unindent();
@@ -463,9 +464,10 @@ QString Printer::Private::classImplementation(const Class &classObject, bool nes
     }
 
     // Generate nested class functions
-    if (!classObject.nestedClasses().isEmpty()) {
-        foreach (Class nestedClass, classObject.nestedClasses()) {
-            code += classImplementation(nestedClass, true);
+    const auto nestedClasses = classObject.nestedClasses();
+    if (!nestedClasses.isEmpty()) {
+        for (const Class &nested : nestedClasses) {
+            code += classImplementation(nested, true);
         }
     }
 
@@ -591,7 +593,7 @@ QString Printer::functionSignature(const Function &function, const QString &clas
     s += '(';
     if (function.hasArguments()) {
         QStringList arguments;
-        foreach (Function::Argument argument, function.arguments()) {
+        for (Function::Argument &argument : function.arguments()) {
             if (!forImplementation) {
                 arguments.append(argument.headerDeclaration());
             } else {
@@ -688,16 +690,7 @@ void Printer::printHeader(const File &file)
     className = headerInfo.fileName(); // remove path, keep only filename
     className.replace('-', "_");
 
-    QString includeGuard;
-    if (!file.nameSpace().isEmpty())
-        includeGuard += file.nameSpace().toUpper() + '_';
-
-    includeGuard += className.toUpper();
-    includeGuard.replace('.', "_");
-
-    out += "#ifndef " + includeGuard;
-    out += "#define " + includeGuard;
-
+    out.addLine(QStringLiteral("#pragma once"));
     out.newLine();
 
     // Create includes
@@ -731,8 +724,7 @@ void Printer::printHeader(const File &file)
     Enum::List enums = file.fileEnums();
     Enum::List::ConstIterator enumIt;
     for (enumIt = enums.constBegin(); enumIt != enums.constEnd(); ++enumIt) {
-        out += (*enumIt).declaration();
-        out.newLine();
+        (*enumIt).printDeclaration(out);
     }
 
     // Create forward declarations
@@ -740,9 +732,14 @@ void Printer::printHeader(const File &file)
     Class::List::ConstIterator it;
     for (it = classes.constBegin(); it != classes.constEnd(); ++it) {
         const QStringList decls = (*it).forwardDeclarations();
+#if QT_VERSION > QT_VERSION_CHECK(5, 14, 0)
+        auto newProcessed = QSet<QString>(decls.begin(), decls.end());
+        processed += newProcessed;
+#else
         processed += decls.toSet();
+#endif
     }
-    QStringList fwdClasses = processed.toList();
+    auto fwdClasses = processed.values();
     fwdClasses.sort();
     fwdClasses += QString(); // for proper closing of the namespace blocks below
 
@@ -796,9 +793,6 @@ void Printer::printHeader(const File &file)
         out.newLine();
     }
 
-    // Finish file
-    out += "#endif";
-
     // Print to file
     QString filename = file.filenameHeader();
 
@@ -945,41 +939,33 @@ void Printer::printImplementation(const File &file, bool createHeaderInclude)
 void Printer::Private::printCodeIntoFile(const Code &code, QFile *file)
 {
     const QString outText = code.text();
-    bool identical = true;
-    if (file->exists()) {
+
+    static bool s_compareOutput = qEnvironmentVariableIsSet("LIBKODE_COMPARE_OUTPUT");
+
+    bool identical = false;
+    if (s_compareOutput && file->exists()) {
         if (!file->open(QIODevice::ReadOnly)) {
             qWarning("Can't open '%s' for reading.", qPrintable(file->fileName()));
             return;
         }
 
         QTextStream fileReaderStream(file);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         fileReaderStream.setCodec(QTextCodec::codecForName("UTF-8"));
+#endif
 
         QTextStream codeStream(outText.toUtf8());
         QString fileLine, outLine;
-#if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
+        identical = true;
         while (fileReaderStream.readLineInto(&fileLine) && codeStream.readLineInto(&outLine)) {
             if (fileLine != outLine) {
                 identical = false;
                 break;
             }
         }
-#else
-        while (!fileReaderStream.atEnd() && !codeStream.atEnd()) {
-            fileLine = fileReaderStream.readLine();
-            outLine = codeStream.readLine();
-            if (fileLine != outLine) {
-                identical = false;
-                break;
-            }
-        }
-#endif
-
         if (identical)
             identical = fileReaderStream.atEnd() && codeStream.atEnd();
         file->close();
-    } else {
-        identical = false;
     }
 
     if (!identical) {
@@ -989,7 +975,9 @@ void Printer::Private::printCodeIntoFile(const Code &code, QFile *file)
         }
 
         QTextStream fileWriterStream(file);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         fileWriterStream.setCodec(QTextCodec::codecForName("UTF-8"));
+#endif
         fileWriterStream << outText;
 
         file->close();
