@@ -2,7 +2,7 @@
 **
 ** This file is part of the KD Soap library.
 **
-** SPDX-FileCopyrightText: 2010-2020 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
+** SPDX-FileCopyrightText: 2010-2021 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 **
 ** SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDAB-KDSoap OR LicenseRef-KDAB-KDSoap-US
 **
@@ -21,12 +21,13 @@
 #include "KDSoapReplySslHandler_p.h"
 #include "KDSoapSslHandler.h"
 
-KDSoapReplySslHandler::KDSoapReplySslHandler(QNetworkReply *reply, KDSoapSslHandler *handler) :
-    QObject(reply), m_handler(handler)
+KDSoapReplySslHandler::KDSoapReplySslHandler(QNetworkReply *reply, KDSoapSslHandler *handler)
+    : QObject(reply)
+    , m_handler(handler)
 {
     Q_ASSERT(reply);
     Q_ASSERT(handler);
-    QObject::connect(reply, SIGNAL(sslErrors(QList<QSslError>)), this, SLOT(slotReplySslErrors(QList<QSslError>)));
+    QObject::connect(reply, &QNetworkReply::sslErrors, this, &KDSoapReplySslHandler::slotReplySslErrors);
 }
 
 void KDSoapReplySslHandler::slotReplySslErrors(const QList<QSslError> &errors)
