@@ -2,7 +2,7 @@
 **
 ** This file is part of the KD Soap library.
 **
-** SPDX-FileCopyrightText: 2010-2020 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
+** SPDX-FileCopyrightText: 2010-2021 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 **
 ** SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDAB-KDSoap OR LicenseRef-KDAB-KDSoap-US
 **
@@ -27,9 +27,8 @@
 #endif
 
 // Helper for xmlBufferCompare
-static bool textBufferCompare(
-    const QByteArray &source, const QByteArray &dest,  // for the qDebug only
-    QIODevice &sourceFile, QIODevice &destFile)
+static bool textBufferCompare(const QByteArray &source, const QByteArray &dest, // for the qDebug only
+                              QIODevice &sourceFile, QIODevice &destFile)
 {
     int lineNumber = 1;
     while (!sourceFile.atEnd()) {
@@ -51,29 +50,12 @@ static bool textBufferCompare(
     return true;
 }
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-#if QT_VERSION < QT_VERSION_CHECK(5, 6, 0)
-QT_BEGIN_NAMESPACE
-// Avoid QHash randomization so that the order of the XML attributes is stable
-extern Q_CORE_EXPORT QBasicAtomicInt qt_qhash_seed; // from qhash.cpp
-QT_END_NAMESPACE
-#endif
-
 static void initHashSeed()
 {
-#if QT_VERSION < QT_VERSION_CHECK(5, 6, 0)
-    // If the seed not initialized yet (-1), set it to 0;
-    // if it was 0 already, do nothing;
-    // otherwise abort, so we don't get unexplained test failures later.
-    qt_qhash_seed.testAndSetRelaxed(-1, 0);
-    Q_ASSERT(qt_qhash_seed.loadAcquire() == 0);
-#else
     qSetGlobalQHashSeed(0);
-#endif
 }
 
 Q_CONSTRUCTOR_FUNCTION(initHashSeed)
-#endif
 
 // A tool for comparing XML documents and outputting something useful if they differ
 bool KDSoapUnitTestHelpers::xmlBufferCompare(const QByteArray &source, const QByteArray &dest)
@@ -123,16 +105,16 @@ void KDSoapUnitTestHelpers::httpGet(const QUrl &url)
     QNetworkRequest request(url);
     QNetworkAccessManager manager;
     QNetworkReply *reply = manager.get(request);
-    //QObject::connect(reply, SIGNAL(sslErrors(QList<QSslError>)), reply, SLOT(ignoreSslErrors()));
+    // QObject::connect(reply, SIGNAL(sslErrors(QList<QSslError>)), reply, SLOT(ignoreSslErrors()));
 
     QEventLoop ev;
-    QObject::connect(reply, SIGNAL(finished()), &ev, SLOT(quit()));
+    QObject::connect(reply, &QNetworkReply::finished, &ev, &QEventLoop::quit);
     ev.exec();
 
-    //QObject::connect(reply, SIGNAL(finished()), &QTestEventLoop::instance(), SLOT(exitLoop()));
-    //QTestEventLoop::instance().enterLoop(11);
+    // QObject::connect(reply, SIGNAL(finished()), &QTestEventLoop::instance(), SLOT(exitLoop()));
+    // QTestEventLoop::instance().enterLoop(11);
 
-    //qDebug() << "httpGet:" << reply->readAll();
+    // qDebug() << "httpGet:" << reply->readAll();
     delete reply;
 }
 
@@ -143,7 +125,7 @@ void KDSoapUnitTestHelpers::httpGet(const QUrl &url)
 static void setupSslServer(QSslSocket *serverSocket)
 {
     Q_INIT_RESOURCE(testtools);
-    //qDebug() << "setupSslServer";
+    // qDebug() << "setupSslServer";
     serverSocket->setProtocol(QSsl::AnyProtocol);
     serverSocket->setLocalCertificate(QString::fromLatin1(":/certs/test-127.0.0.1-cert.pem"));
     serverSocket->setPrivateKey(QString::fromLatin1(":/certs/test-127.0.0.1-key.pem"));
@@ -160,22 +142,7 @@ bool KDSoapUnitTestHelpers::setSslConfiguration()
 
     // To make SSL work, we need to tell Qt about our local certificate
 
-    // Both ways work:
-#if 0
     QSslConfiguration defaultConfig = QSslConfiguration::defaultConfiguration();
-    QFile certFile(QString::fromLatin1(":/certs/cacert.pem"));
-    if (!certFile.open(QIODevice::ReadOnly)) {
-        qDebug() << "Could not open cacert.pem";
-        return false;
-    }
-    QSslCertificate cert(&certFile);
-    if (!cert.isValid()) {
-        return false;
-    }
-    defaultConfig.setCaCertificates(QList<QSslCertificate>() << cert);
-    QSslConfiguration::setDefaultConfiguration(defaultConfig);
-#endif
-
     QFile certFile(QString::fromLatin1(":/certs/cacert.pem"));
     if (!certFile.open(QIODevice::ReadOnly)) {
         qDebug() << "Could not open cacert.pem";
@@ -183,13 +150,13 @@ bool KDSoapUnitTestHelpers::setSslConfiguration()
     }
     QSslCertificate cert(&certFile);
     const QDateTime currentTime = QDateTime::currentDateTime();
-    if (cert.effectiveDate() > currentTime
-            || cert.expiryDate() < currentTime) {
+    if (cert.effectiveDate() > currentTime || cert.expiryDate() < currentTime) {
         qDebug() << "Certificate" << certFile.fileName() << "is not valid";
         qDebug() << "It is valid from" << cert.effectiveDate() << "to" << cert.expiryDate();
         return false;
     }
-    QSslSocket::addDefaultCaCertificate(cert);
+    defaultConfig.setCaCertificates({ cert });
+    QSslConfiguration::setDefaultConfiguration(defaultConfig);
 
     return true;
 }
@@ -200,46 +167,41 @@ class BlockingHttpServer : public QTcpServer
 {
     Q_OBJECT
 public:
-    BlockingHttpServer(bool ssl) : doSsl(ssl), sslSocket(0) {}
-    ~BlockingHttpServer() {}
+    BlockingHttpServer(bool ssl)
+        : doSsl(ssl)
+        , sslSocket(0)
+    {
+    }
+    ~BlockingHttpServer()
+    {
+    }
 
     QTcpSocket *waitForNextConnectionSocket()
     {
         if (!waitForNewConnection(20000)) { // 2000 would be enough, except in valgrind
             return 0;
         }
-        if (doSsl) {
-            Q_ASSERT(sslSocket);
-            return sslSocket;
-        } else {
-            //qDebug() << "returning nextPendingConnection";
-            return nextPendingConnection();
-        }
+        return nextPendingConnection();
     }
 
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
     virtual void incomingConnection(qintptr socketDescriptor) override
-#else
-    virtual void incomingConnection(int socketDescriptor) override
-#endif
     {
 #ifndef QT_NO_OPENSSL
         if (doSsl) {
             QSslSocket *serverSocket = new QSslSocket;
             serverSocket->setParent(this);
             serverSocket->setSocketDescriptor(socketDescriptor);
-            connect(serverSocket, SIGNAL(sslErrors(QList<QSslError>)), this, SLOT(slotSslErrors(QList<QSslError>)));
+            connect(serverSocket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &BlockingHttpServer::slotSslErrors);
             setupSslServer(serverSocket);
-            //qDebug() << "Created QSslSocket, starting server encryption";
+            // qDebug() << "Created QSslSocket, starting server encryption";
             serverSocket->startServerEncryption();
             sslSocket = serverSocket;
-            // If startServerEncryption fails internally [and waitForEncrypted hangs],
+            // If startServerEncryption fails internally,
             // then this is how to debug it.
             // A way to catch such errors is really missing in Qt..
-            //qDebug() << "startServerEncryption said:" << sslSocket->errorString();
-            bool ok = serverSocket->waitForEncrypted();
-            Q_ASSERT(ok);
-            Q_UNUSED(ok);
+            // qDebug() << "startServerEncryption said:" << sslSocket->errorString();
+            serverSocket->waitForEncrypted();
+            addPendingConnection(serverSocket);
         } else
 #endif
             QTcpServer::incomingConnection(socketDescriptor);
@@ -258,6 +220,7 @@ private slots:
         Q_UNUSED(errors);
 #endif
     }
+
 private:
     bool doSsl;
     QTcpSocket *sslSocket;
@@ -305,7 +268,7 @@ static HeadersMap parseHeaders(const QByteArray &headerData)
         }
         const QByteArray header = line.left(pos);
         const QByteArray value = line.mid(pos + 1).trimmed(); // remove space before and \r\n after
-        //qDebug() << "HEADER" << header << "VALUE" << value;
+        // qDebug() << "HEADER" << header << "VALUE" << value;
         headersMap.insert(header, value);
     }
     return headersMap;
@@ -344,13 +307,13 @@ void HttpServerThread::run()
         return;
     }
 
-    Q_FOREVER {
+    Q_FOREVER
+    {
         // get the "request" packet
         if (doDebug) {
             qDebug() << "HttpServerThread: waiting for read";
         }
-        if (clientSocket->state() == QAbstractSocket::UnconnectedState ||
-                !clientSocket->waitForReadyRead(2000)) {
+        if (clientSocket->state() == QAbstractSocket::UnconnectedState || !clientSocket->waitForReadyRead(2000)) {
             if (clientSocket->state() == QAbstractSocket::UnconnectedState) {
                 delete clientSocket;
                 if (doDebug) {
@@ -373,7 +336,7 @@ void HttpServerThread::run()
         lock.relock();
         const bool splitOK = splitHeadersAndData(request, m_receivedHeaders, m_receivedData);
         if (!splitOK) {
-            //if (doDebug)
+            // if (doDebug)
             //    qDebug() << "Storing partial request" << request;
             m_partialRequest = request;
             continue;
@@ -382,7 +345,7 @@ void HttpServerThread::run()
         m_headers = parseHeaders(m_receivedHeaders);
 
         if (m_headers.value("Content-Length").toInt() > m_receivedData.size()) {
-            //if (doDebug)
+            // if (doDebug)
             //    qDebug() << "Storing partial request" << request;
             m_partialRequest = request;
             continue;
@@ -391,42 +354,69 @@ void HttpServerThread::run()
         m_partialRequest.clear();
 
         if (m_headers.value("_path").endsWith("terminateThread")) { // we're asked to exit
-            break;    // normal exit
+            break; // normal exit
         }
 
-        // TODO compared with expected SoapAction
         QList<QByteArray> contentTypes = m_headers.value("Content-Type").split(';');
-        if (contentTypes[0] == "text/xml" && m_headers.value("SoapAction").isEmpty()) {
-            qDebug() << "ERROR: no SoapAction set for Soap 1.1";
-            break;
-        } else if (contentTypes[0] == "application/soap+xml" && !contentTypes[2].startsWith("action")) {
-            qDebug() << "ERROR: no SoapAction set for Soap 1.2";
-            break;
+        if (contentTypes[0] == "text/xml") {
+            if (m_headers.value("SoapAction").isEmpty()) {
+                qDebug() << "ERROR: no SoapAction set for Soap 1.1";
+                break;
+            }
+
+            if (!m_expectedSoapAction.isEmpty() && m_headers.value("SoapAction") != m_expectedSoapAction) {
+                qDebug("ERROR: Client sent SoapAction HTTP header (\"%s\") which does not match the expected (\"%s\")",
+                       m_headers.value("SoapAction").constData(), m_expectedSoapAction.constData());
+                break;
+            }
+        } else if (m_clientSendsActionInHttpHeader
+                   && contentTypes[0] == "application/soap+xml") {
+            if (!contentTypes[2].startsWith("action")) {
+                qDebug() << "ERROR: no SoapAction set for Soap 1.2";
+                break;
+            }
+
+            QList<QByteArray> actionParts = contentTypes[2].split('=');
+            if (actionParts.length() < 2) {
+                qDebug("ERROR: The action parameter is malformed in the HTTP Content-type header: \"%s\"",
+                       contentTypes[2].constData());
+                break;
+            }
+
+            if (!m_expectedSoapAction.isEmpty()) {
+                actionParts = actionParts[1].split(';');
+                if (actionParts[0] != m_expectedSoapAction) {
+                    qDebug("ERROR: The 'action' parameter which was sent in the HTTP Content-type header (\"%s\") "
+                           "does not match the expected SOAP action: \"%s\"",
+                           m_headers.value("SoapAction").constData(), m_expectedSoapAction.constData());
+                    break;
+                }
+            }
         }
         lock.unlock();
 
-        //qDebug() << "headers received:" << m_receivedHeaders;
-        //qDebug() << headers;
-        //qDebug() << "data received:" << m_receivedData;
+        // qDebug() << "headers received:" << m_receivedHeaders;
+        // qDebug() << headers;
+        // qDebug() << "data received:" << m_receivedData;
 
         if (m_features & BasicAuth) {
             QByteArray authValue = m_headers.value("Authorization");
             if (authValue.isEmpty()) {
-                authValue = m_headers.value("authorization");    // as sent by Qt-4.5
+                authValue = m_headers.value("authorization"); // as sent by Qt-4.5
             }
             bool authOk = false;
             if (!authValue.isEmpty()) {
-                //qDebug() << "got authValue=" << authValue; // looks like "Basic <base64 of user:pass>"
+                // qDebug() << "got authValue=" << authValue; // looks like "Basic <base64 of user:pass>"
                 Method method;
                 QString headerVal;
                 parseAuthLine(QString::fromLatin1(authValue.data(), authValue.size()), &method, &headerVal);
-                //qDebug() << "method=" << method << "headerVal=" << headerVal;
+                // qDebug() << "method=" << method << "headerVal=" << headerVal;
                 switch (method) {
                 case None: // we want auth, so reject "None"
                     break;
                 case Basic: {
                     const QByteArray userPass = QByteArray::fromBase64(headerVal.toLatin1());
-                    //qDebug() << userPass;
+                    // qDebug() << userPass;
                     // TODO if (validateAuth(userPass)) {
                     if (userPass == ("kdab:testpass")) {
                         authOk = true;
@@ -440,7 +430,8 @@ void HttpServerThread::run()
 
             if (!authOk) {
                 // send auth request (Qt supports basic, ntlm and digest)
-                const QByteArray unauthorized = "HTTP/1.1 401 Authorization Required\r\nWWW-Authenticate: Basic realm=\"example\"\r\nContent-Length: 0\r\n\r\n";
+                const QByteArray unauthorized =
+                    "HTTP/1.1 401 Authorization Required\r\nWWW-Authenticate: Basic realm=\"example\"\r\nContent-Length: 0\r\n\r\n";
                 clientSocket->write(unauthorized);
                 if (!clientSocket->waitForBytesWritten(2000)) {
                     qDebug() << "HttpServerThread:" << clientSocket->error() << "writing auth request";
@@ -467,16 +458,48 @@ void HttpServerThread::run()
     }
 }
 
+QByteArray HttpServerThread::expectedSoapAction() const
+{
+    return m_expectedSoapAction;
+}
+
+void HttpServerThread::setExpectedSoapAction(const QByteArray &expectedSoapAction)
+{
+    m_expectedSoapAction = expectedSoapAction;
+}
+
+bool HttpServerThread::clientSendsActionInHttpHeader() const
+{
+    return m_clientSendsActionInHttpHeader;
+}
+
+void HttpServerThread::setClientSendsActionInHttpHeader(bool clientUseWSAddressing)
+{
+    m_clientSendsActionInHttpHeader = clientUseWSAddressing;
+}
+
 const char *KDSoapUnitTestHelpers::xmlEnvBegin11()
+{
+    return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
+           "<soap:Envelope"
+           " xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\""
+           " xmlns:soap-enc=\"http://schemas.xmlsoap.org/soap/encoding/\""
+           " xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\""
+           " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"";
+}
+
+const char *KDSoapUnitTestHelpers::xmlEnvBegin11WithWSAddressing()
 {
     return  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
             "<soap:Envelope"
             " xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\""
             " xmlns:soap-enc=\"http://schemas.xmlsoap.org/soap/encoding/\""
             " xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\""
-            " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"";
+            " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
+            " xmlns:wsa=\"http://www.w3.org/2005/08/addressing\"";
 }
 
+
 const char *KDSoapUnitTestHelpers::xmlEnvBegin12()
 {
     return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
@@ -487,6 +510,18 @@ const char *KDSoapUnitTestHelpers::xmlEnvBegin12()
            " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"";
 }
 
+const char *KDSoapUnitTestHelpers::xmlEnvBegin12WithWSAddressing()
+{
+    return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
+           "<soap:Envelope"
+           " xmlns:soap=\"http://www.w3.org/2003/05/soap-envelope\""
+           " xmlns:soap-enc=\"http://www.w3.org/2003/05/soap-encoding\""
+           " xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\""
+           " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
+           " xmlns:wsa=\"http://www.w3.org/2005/08/addressing\""
+           " xmlns:n1=\"http://www.kdab.com/xml/MyWsdl/\"";
+}
+
 const char *KDSoapUnitTestHelpers::xmlEnvEnd()
 {
     return "</soap:Envelope>";
