@@ -1,6 +1,6 @@
 ## This file is part of the KD Soap library.
 ##
-## SPDX-FileCopyrightText: 2017-2020 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
+## SPDX-FileCopyrightText: 2017-2021 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 ##
 ## SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDAB-KDSoap OR LicenseRef-KDAB-KDSoap-US
 ##
@@ -34,24 +34,3 @@ EOF
 git add $subdir/CMakeLists.txt
 echo "add_subdirectory($subdir)" >> CMakeLists.txt
 
-# QMake
-# Note: do NOT set TARGET, runTest.bat assumes exename==dirname
-
-cat > $subdir/${subdir}.pro <<EOF
-include( \$\${TOP_SOURCE_DIR}/unittests/unittests.pri )
-QT += network xml
-SOURCES = $cppfiles
-test.target = test
-test.commands = ./\$(TARGET)
-test.depends = \$(TARGET)
-QMAKE_EXTRA_TARGETS += test
-
-KDWSDL = ${wsdlfiles}
-
-OTHER_FILES = \$\$KDWSDL
-LIBS        += -L\$\${TOP_BUILD_DIR}/lib
-EOF
-
-git add $subdir/${subdir}.pro
-perl -pi -e '$_ .= "  '$subdir' \\\n" if (/vidyo/)' unittests.pro
-
