@@ -2,7 +2,7 @@
 **
 ** This file is part of the KD Soap library.
 **
-** SPDX-FileCopyrightText: 2010-2020 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
+** SPDX-FileCopyrightText: 2010-2021 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 **
 ** SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDAB-KDSoap OR LicenseRef-KDAB-KDSoap-US
 **
@@ -66,33 +66,36 @@ private Q_SLOTS:
         service.resolveNames(req);
 
         // Check what we sent
-        QByteArray expectedRequestXml =
-            QByteArray(xmlEnvBegin11()) + " xmlns:n1=\"http://schemas.microsoft.com/exchange/services/2006/messages\">"
-            "<soap:Header>"
-            "<n2:ExchangeImpersonation xmlns:n2=\"http://schemas.microsoft.com/exchange/services/2006/types\"><n2:ConnectingSID>"
-            "<n2:PrincipalName>dfaure</n2:PrincipalName>"
-            "<n2:SID>sid</n2:SID>"
-            "<n2:PrimarySmtpAddress>david.faure@kdab.com</n2:PrimarySmtpAddress>"
-            "</n2:ConnectingSID></n2:ExchangeImpersonation>"
-            "<n2:RequestServerVersion xmlns:n2=\"http://schemas.microsoft.com/exchange/services/2006/types\" Version=\"Exchange2007_SP1\"/>"
-            "</soap:Header>"
-            "<soap:Body>"
-            "<n1:ResolveNames ReturnFullContactData=\"true\">"
-            "<n1:ParentFolderIds><n3:FolderId xmlns:n3=\"http://schemas.microsoft.com/exchange/services/2006/types\" Id=\"folderId\"/></n1:ParentFolderIds>"
-            "<n1:UnresolvedEntry/>"
-            "</n1:ResolveNames>"
-            "</soap:Body>" + xmlEnvEnd()
-            + '\n'; // added by QXmlStreamWriter::writeEndDocument
+        QByteArray expectedRequestXml = QByteArray(xmlEnvBegin11())
+            + " xmlns:n1=\"http://schemas.microsoft.com/exchange/services/2006/messages\">"
+              "<soap:Header>"
+              "<n2:ExchangeImpersonation xmlns:n2=\"http://schemas.microsoft.com/exchange/services/2006/types\"><n2:ConnectingSID>"
+              "<n2:PrincipalName>dfaure</n2:PrincipalName>"
+              "<n2:SID>sid</n2:SID>"
+              "<n2:PrimarySmtpAddress>david.faure@kdab.com</n2:PrimarySmtpAddress>"
+              "</n2:ConnectingSID></n2:ExchangeImpersonation>"
+              "<n2:RequestServerVersion xmlns:n2=\"http://schemas.microsoft.com/exchange/services/2006/types\" Version=\"Exchange2007_SP1\"/>"
+              "</soap:Header>"
+              "<soap:Body>"
+              "<n1:ResolveNames ReturnFullContactData=\"true\">"
+              "<n1:ParentFolderIds><n3:FolderId xmlns:n3=\"http://schemas.microsoft.com/exchange/services/2006/types\" "
+              "Id=\"folderId\"/></n1:ParentFolderIds>"
+              "<n1:UnresolvedEntry/>"
+              "</n1:ResolveNames>"
+              "</soap:Body>"
+            + xmlEnvEnd() + '\n'; // added by QXmlStreamWriter::writeEndDocument
         QVERIFY(xmlBufferCompare(server.receivedData(), expectedRequestXml));
     }
 
 private:
     static QByteArray queryResponse()
     {
-        return QByteArray(xmlEnvBegin11()) + " xmlns:sf=\"urn:sobject.partner.soap.sforce.com\"><soap:Body>"
-               "<queryResponse>" // TODO
-               "</queryResponse>"
-               "</soap:Body>" + xmlEnvEnd();
+        return QByteArray(xmlEnvBegin11())
+            + " xmlns:sf=\"urn:sobject.partner.soap.sforce.com\"><soap:Body>"
+              "<queryResponse>" // TODO
+              "</queryResponse>"
+              "</soap:Body>"
+            + xmlEnvEnd();
     }
 };
 
